/* callbacks.c                                           */
/*                                                       */
/* The way arrow keys currently work (Dec 97) for focus  */
/* control is not good enough for a form type of window. */
/* This example program gets around this by handling the */
/* arrow key entirely using the code given below.        */
/*                                                       */
/* We basically make a table of the widgets that are in  */
/* our window.  Each entry in the table has five         */
/* members, the current widget and which widgets to give */
/* focus for each of the four possible directions (up,   */
/* down, left, right.)  So if the down arrow key is      */
/* pressed, the widget that currently has focus is       */
/* looked up in the table.  The widget that is indicated */
/* as being the 'down' one is then given focus.          */
/*                                                       */
/* To make use of this, you simply have to create a      */
/* table of type, focus_ctrl_t for each window.  Fill    */
/* them in (as is done in base_setup() below.)  Then,    */
/* attach a Pt_CB_RAW callback to each window.  In the   */
/* callback, simply call change_focus_filter(), which    */
/* is also below.                                        */
/*                                                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.13A */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "proto.h"

/*
 * focus_ctrl_t defines which widget to go to when wname has the focus and
 * an arrow key arrives.
 */
typedef struct {
    int wname;      /* widget we are arrow keying from */
    int up;         /* if up arrowing then give this widget the focus */
    int down;       /* if down arrowing then give this widget the focus */
    int left;       /* if left arrowing then give this widget the focus */
    int right;      /* if right arrowing then give this widget the focus */
} focus_ctrl_t;

focus_ctrl_t    base_focus[11]; /* for the base window */

static PtWidget_t *change_focus_filter( PtWidget_t *window, focus_ctrl_t *ctrl, PhEvent_t *event );

/*----------------------------------------------------------------------
 *
 * base_setup
 *
 * Pre-realize setup function for the base window.
 *
 *---------------------------------------------------------------------*/
int
base_setup( PtWidget_t *link_instance, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
    base_focus[0].wname = ABN_base_name_txt;
    base_focus[0].up    = ABN_base_close_btn;
    base_focus[0].down  = ABN_base_street_txt;
    base_focus[0].left  = ABN_base_phone_txt;
    base_focus[0].right = ABN_base_phone_txt;

    base_focus[1].wname = ABN_base_phone_txt;
    base_focus[1].up    = ABN_base_cancel_btn;
    base_focus[1].down  = ABN_base_A_tgl;
    base_focus[1].left  = ABN_base_name_txt;
    base_focus[1].right = ABN_base_name_txt;

    base_focus[2].wname = ABN_base_street_txt;
    base_focus[2].up    = ABN_base_name_txt;
    base_focus[2].down  = ABN_base_notes_txt;
    base_focus[2].left  = ABN_base_B_tgl;
    base_focus[2].right = ABN_base_A_tgl;

    base_focus[3].wname = ABN_base_A_tgl;
    base_focus[3].up    = ABN_base_phone_txt;
    base_focus[3].down  = ABN_base_notes_txt;
    base_focus[3].left  = ABN_base_street_txt;
    base_focus[3].right = ABN_base_B_tgl;

    base_focus[4].wname = ABN_base_B_tgl;
    base_focus[4].up    = ABN_base_phone_txt;
    base_focus[4].down  = ABN_base_notes_txt;
    base_focus[4].left  = ABN_base_A_tgl;
    base_focus[4].right = ABN_base_street_txt;

    base_focus[5].wname = ABN_base_notes_txt;
    base_focus[5].up    = ABN_base_street_txt;
    base_focus[5].down  = ABN_base_apply_btn;
    base_focus[5].left  = ABN_base_notes_txt;
    base_focus[5].right = ABN_base_notes_txt;

    base_focus[6].wname = ABN_base_apply_btn;
    base_focus[6].up    = ABN_base_notes_txt;
    base_focus[6].down  = ABN_base_close_btn;
    base_focus[6].left  = ABN_base_default_btn;
    base_focus[6].right = ABN_base_save_btn;
    
    base_focus[7].wname = ABN_base_save_btn;
    base_focus[7].up    = ABN_base_notes_txt;
    base_focus[7].down  = ABN_base_close_btn;
    base_focus[7].left  = ABN_base_apply_btn;
    base_focus[7].right = ABN_base_default_btn;

    base_focus[8].wname = ABN_base_default_btn;
    base_focus[8].up    = ABN_base_notes_txt;
    base_focus[8].down  = ABN_base_cancel_btn;
    base_focus[8].left  = ABN_base_save_btn;
    base_focus[8].right = ABN_base_apply_btn;

    base_focus[9].wname = ABN_base_close_btn;
    base_focus[9].up    = ABN_base_apply_btn;
    base_focus[9].down  = ABN_base_name_txt;
    base_focus[9].left  = ABN_base_cancel_btn;
    base_focus[9].right = ABN_base_cancel_btn;

    base_focus[10].wname = ABN_base_cancel_btn; 
    base_focus[10].up    = ABN_base_default_btn;
    base_focus[10].down  = ABN_base_phone_txt;
    base_focus[10].left  = ABN_base_close_btn;
    base_focus[10].right = ABN_base_close_btn;

    base_focus[11].wname = -1;

	/* eliminate 'unreferenced' warnings */
	link_instance = link_instance, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );
}

/*----------------------------------------------------------------------
 *
 * base_raw_cb
 *
 * Pt_CB_RAW callback attached to the base window.  This is set up to
 * be called whenever a Ph_EV_KEY event arrives.
 *
 *---------------------------------------------------------------------*/
int
base_raw_cb( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
    change_focus_filter(ABW_base, base_focus, cbinfo->event);

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );
}

/*----------------------------------------------------------------------
 *
 * change_focus_filter
 *
 * This will figure out which widget currently has focus.  Based on that,
 * this will look up that widget in the given focus control table, ctrl.
 * Depending on which arrow key was pressed, this will then change the
 * focus to be the widget that is in the indicated direction in the
 * focus control entry.  For example, if the down arrow was pressed then
 * the widget, ctrl[i].down, will get focus next.
 *
 *---------------------------------------------------------------------*/
static PtWidget_t *
change_focus_filter( PtWidget_t *window, focus_ctrl_t *ctrl, PhEvent_t *event )
{
    int             i = 0, focusn;
    long            *flags;
    PhKeyEvent_t    *kdata;
    PtArg_t         arg;
    PtWidget_t      *focusw;

    if (event->type != Ph_EV_KEY)
        return NULL;
    
    /* find out which widget currently has focus */
    
    if ((focusw = PtContainerFindFocus(window)) == NULL)
        return NULL;
    /* Check if it is a procreated widget, i.e. if it is a procreated child
       of a compound widget.  If it is, find the real parent. */
    PtSetArg( &arg, Pt_ARG_FLAGS, &flags, 0 );
    while (1) {
        PtGetResources( focusw, 1, &arg );
        if ( *flags & Pt_PROCREATED )
            focusw = PtWidgetParent(focusw);
        else
            break;
    }
    
    /* get the key that was pressed, we'll accept arrow keys and combinations
       of modifiers and arrow keys */
       
    kdata = PhGetData( event );
    if ( !((kdata->key_flags & (Pk_KF_Key_Down|Pk_KF_Key_Repeat)) &&
           (kdata->key_flags & (Pk_KF_Sym_Valid|Pk_KF_Cap_Valid))) )
        /* must be key down or key repeat and must have a key_cap or key_sym */
        return NULL;
    if ( !(kdata->key_flags & Pk_KF_Sym_Valid) )
        kdata->key_sym = kdata->key_cap;
    /* we have a valid key_sym */
    if (!(kdata->key_sym == Pk_Up   || kdata->key_sym == Pk_Down ||
          kdata->key_sym == Pk_Left || kdata->key_sym == Pk_Right))
        return NULL;
        
    /* search the focus control table to the widget that has focus */
    
    focusn = ApName(focusw); /* returns the ABN_ value */
    while (ctrl[i].wname != -1) {
        if ( focusn == ctrl[i].wname )
            break;
        i++;
    }
    if ( ctrl[i].wname == -1 )
        return NULL;
        
    /* figure out which widget gets focus, given the arrow key that was
       pressed, and give it focus */
       
    switch (kdata->key_sym) {
    case Pk_Up:
        focusn = ctrl[i].up;
        break;
    case Pk_Down:
        focusn = ctrl[i].down;
        break;
    case Pk_Left:
        focusn = ctrl[i].left;
        break;
    case Pk_Right:
        focusn = ctrl[i].right;
        break;
    }
    focusw = ApGetWidgetPtr( window, focusn ); /* returns the widget pointer */
    PtContainerGiveFocus( focusw, NULL );
    return focusw;
}
